@@ -344,6 +344,11 @@ load (const char *file)
 		case FirstBrace:
 		case KeyDef:
 		case KeyDefOnKey:
+			if (c == '#') {
+				state = IgnoreToEOL;
+				break;
+			}
+
 			if (c == '[' && state != KeyDefOnKey){
 				TSecHeader *temp;
 		
@@ -512,6 +517,13 @@ access_config_extended (access_type mode, const char *section_name,
 	const char *ret_val;
 	gboolean internal_def;
 
+	static time_t cache_time = 0;
+	static char *cache_filename_key = NULL;
+	static char *cache_filename_value1 = NULL;
+	static char *cache_filename_value2 = NULL;
+	gboolean cache_valid;
+	time_t now;
+
 	switch (mode) {
 	case SET:
 		/* fall through to normal behaviour */
@@ -519,12 +531,32 @@ access_config_extended (access_type mode, const char *section_name,
 		ret_val = access_config (mode, section_name, key_name, def,
 					 filename, def_used);
 		g_free(filename);
+ 		cache_time = 0;  /* Invalidate cache.  */
 		return ret_val;
 	case LOOKUP:
-		/* check the system wide override config tree first */
-		tmp = g_concat_dir_and_file ("gnome/config-override",rel_file);
-		filename = gnome_config_file (tmp);
-		g_free (tmp);
+ 		now = time (NULL);
+ 		cache_valid = (cache_filename_key &&
+ 			       strcmp (cache_filename_key, rel_file) == 0 &&
+ 			       now - cache_time <= 2);
+ 		if (!cache_valid) {
+ 			if (cache_filename_key) g_free (cache_filename_key);
+ 			cache_filename_key = g_strdup (rel_file);
+ 			cache_time = now;
+ 			if (cache_filename_value1) g_free (cache_filename_value1);
+ 			if (cache_filename_value2) g_free (cache_filename_value2);
+ 		}
+ 		
+ 		if (cache_valid) {
+ 			filename = cache_filename_value1
+ 				? g_strdup (cache_filename_value1)
+ 				: NULL;
+ 		} else {
+ 			/* check the system wide override config tree first */
+ 			tmp = g_concat_dir_and_file ("gnome/config-override",rel_file);
+ 			filename = gnome_config_file (tmp);
+ 			g_free (tmp);
+ 			cache_filename_value1 = filename ? g_strdup (filename) : NULL;
+ 		}
 		if (filename) {
 			/* the required config file exists */
 			ret_val = access_config (mode, section_name, key_name,
@@ -547,9 +579,16 @@ access_config_extended (access_type mode, const char *section_name,
 		}
 		g_assert (ret_val == NULL);
 		/* fall through to the system wide config default tree */
-		tmp = g_concat_dir_and_file ("gnome/config", rel_file);
-		filename = gnome_config_file (tmp);
-		g_free (tmp);
+		if (cache_valid) {
+			filename = cache_filename_value2
+				? g_strdup (cache_filename_value2)
+				: NULL;
+		} else {
+			tmp = g_concat_dir_and_file ("gnome/config", rel_file);
+			filename = gnome_config_file (tmp);
+			g_free (tmp);
+			cache_filename_value2 = filename ? g_strdup (filename) : NULL;
+		}
 		if (filename) {
 			/* the file exists */
 			ret_val = access_config (mode, section_name, key_name,
